@@ -49,6 +49,9 @@
 #include <execinfo.h> // for backtrace()
 #endif
 
+/** number of updates of the progress bat per second */
+#define PROGRESS_UPDATES_PER_SECOND 4
+
 /*
    A plugin can be unloaded through two different ways. The possible
    scenarios are:
@@ -75,9 +78,15 @@ Kwave::Plugin::Plugin(const PluginContext &c)
      m_progress(0),
      m_confirm_cancel(0),
      m_usage_count(0),
-     m_usage_lock()
+     m_usage_lock(),
+     m_progress_timer(),
+     m_current_progress(-1),
+     m_progress_lock()
 {
     use();
+
+    connect(&m_progress_timer, SIGNAL(timeout()),
+            this, SLOT(updateProgressTick()));
 }
 
 //***************************************************************************
@@ -143,15 +152,12 @@ int Kwave::Plugin::start(QStringList &)
     QMutexLocker lock(&m_thread_lock);
     m_stop = false;
 
-    Q_ASSERT(!m_progress);
-    Q_ASSERT(!m_confirm_cancel);
-
     // check: start() must be called from the GUI thread only!
     Q_ASSERT(this->thread() == QThread::currentThread());
     Q_ASSERT(this->thread() == qApp->thread());
 
     // create a progress dialog for processing mode (not used for pre-listen)
-    if (m_progress_enabled) {
+    if (m_progress_enabled && !m_progress) {
 	m_progress = new QProgressDialog(parentWidget());
 	Q_ASSERT(m_progress);
     }
@@ -165,21 +171,29 @@ int Kwave::Plugin::start(QStringList &)
 	m_progress->setModal(true);
 	m_progress->setVisible(false);
 	m_progress->setMinimumDuration(2000);
-	m_progress->setAutoClose(true);
+	m_progress->setAutoClose(false);
 	m_progress->setMaximum((last - first + 1) * tracks);
 	m_progress->setValue(0);
 	m_progress->setLabelText(progressText());
 	int h = m_progress->sizeHint().height();
 	int w = m_progress->sizeHint().height();
-	if (w < 4*h) w = 4*h;
+	if (w < 4 * h) w = 4 * h;
 	m_progress->setFixedSize(w, h);
 
 	// use a "proxy" that asks for confirmation of cancel
-	m_confirm_cancel = new ConfirmCancelProxy(m_progress,
+	if (!m_confirm_cancel) {
+	    m_confirm_cancel = new ConfirmCancelProxy(m_progress,
 		0, 0, this, SLOT(cancel()));
-	Q_ASSERT(m_confirm_cancel);
-	connect(m_progress,      SIGNAL(canceled()),
+	    Q_ASSERT(m_confirm_cancel);
+	}
+	connect(m_progress,       SIGNAL(canceled()),
 		m_confirm_cancel, SLOT(cancel()));
+	connect(this,             SIGNAL(setProgressText(const QString &)),
+	        m_progress,       SLOT(setLabelText(const QString &)),
+	        Qt::QueuedConnection);
+	connect(this, SIGNAL(sigDone(Kwave::Plugin *)),
+		this, SLOT(closeProgressDialog(Kwave::Plugin *)),
+		Qt::QueuedConnection);
 	m_progress->setVisible(true);
     }
 
@@ -242,13 +256,54 @@ void Kwave::Plugin::setProgressDialogEnabled(bool enable)
 //***************************************************************************
 void Kwave::Plugin::updateProgress(unsigned int progress)
 {
-//     qDebug("Kwave::Plugin::updateProgress(%u)", progress);
-
     // check: this must be called from the GUI thread only!
     Q_ASSERT(this->thread() == QThread::currentThread());
     Q_ASSERT(this->thread() == qApp->thread());
 
-    if (m_progress) m_progress->setValue(progress);
+//     qDebug("Kwave::Plugin::updateProgress(%u)", progress);
+    QMutexLocker lock(&m_progress_lock);
+
+    // take over the current progress
+    // note: no lock needed, this is called in the GUI thread context!
+    m_current_progress = progress;
+
+    // start the timer for updating the progress bar if it is not active
+    if (!m_progress_timer.isActive()) {
+	m_progress_timer.setSingleShot(true);
+	m_progress_timer.start(1000 / PROGRESS_UPDATES_PER_SECOND);
+    }
+}
+
+//***************************************************************************
+void Kwave::Plugin::updateProgressTick()
+{
+    // check: this must be called from the GUI thread only!
+    Q_ASSERT(this->thread() == QThread::currentThread());
+    Q_ASSERT(this->thread() == qApp->thread());
+
+//     qDebug("void Kwave::Plugin::updateProgressTick(%d)", m_current_progress);
+    QMutexLocker lock(&m_progress_lock);
+    if (m_progress) m_progress->setValue(m_current_progress);
+}
+
+//***************************************************************************
+void Kwave::Plugin::closeProgressDialog(Kwave::Plugin *)
+{
+    // check: this must be called from the GUI thread only!
+    Q_ASSERT(this->thread() == QThread::currentThread());
+    Q_ASSERT(this->thread() == qApp->thread());
+
+    QMutexLocker lock(&m_progress_lock);
+    if (m_confirm_cancel) {
+	ConfirmCancelProxy *proxy = m_confirm_cancel;
+	m_confirm_cancel = 0;
+	delete proxy;
+    }
+    if (m_progress) {
+	QProgressDialog *prog = m_progress;
+	m_progress = 0;
+	delete prog;
+    }
 }
 
 //***************************************************************************
@@ -316,6 +371,15 @@ void Kwave::Plugin::close()
     if (m_thread && m_thread->isRunning() &&
         (QThread::currentThread() != m_thread) )
     {
+	// check: this must be called from the GUI thread only!
+	Q_ASSERT(this->thread() == QThread::currentThread());
+	Q_ASSERT(this->thread() == qApp->thread());
+
+	m_progress_timer.stop();
+	qApp->sendPostedEvents();
+	qApp->processEvents();
+	qApp->syncX();
+
 	stop();
     }
 }
@@ -345,7 +409,7 @@ void Kwave::Plugin::release()
 }
 
 //***************************************************************************
-PluginManager &Kwave::Plugin::manager()
+Kwave::PluginManager &Kwave::Plugin::manager()
 {
     return m_context.m_plugin_manager;
 }
@@ -381,7 +445,7 @@ unsigned int Kwave::Plugin::signalLength()
 }
 
 //***************************************************************************
-qreal Kwave::Plugin::signalRate()
+double Kwave::Plugin::signalRate()
 {
     return manager().signalRate();
 }
@@ -430,7 +494,7 @@ void *Kwave::Plugin::handle()
 }
 
 //***************************************************************************
-QString Kwave::Plugin::zoom2string(qreal percent)
+QString Kwave::Plugin::zoom2string(double percent)
 {
     QString result = "";
 
@@ -451,7 +515,7 @@ QString Kwave::Plugin::zoom2string(qreal percent)
 }
 
 //***************************************************************************
-QString Kwave::Plugin::ms2string(qreal ms, int precision)
+QString Kwave::Plugin::ms2string(double ms, int precision)
 {
     char buf[128];
     int bufsize = 128;
@@ -473,7 +537,7 @@ QString Kwave::Plugin::ms2string(qreal ms, int precision)
 	if (m < 1) {
 	    char format[128];
 	    int digits = static_cast<int>(
-		ceil(static_cast<qreal>(precision+1) - log10(ms)));
+		ceil(static_cast<double>(precision+1) - log10(ms)));
 	    snprintf(format, sizeof(format), "%%0.%df s", digits);
 	    snprintf(buf, bufsize, format, ms / 1000.0);
 	} else {
