@@ -24,6 +24,7 @@
 #include <QObject>
 #include <QString>
 #include <QThread>
+#include <QTimer>
 #include <QVector>
 
 #include <kdemacros.h>
@@ -34,7 +35,6 @@ class ConfirmCancelProxy;
 class FileInfo;
 class MultiTrackReader;
 class MultiTrackWriter;
-class PluginManager;
 class SampleReader;
 class SignalManager;
 class TopWidget;
@@ -42,16 +42,23 @@ class QStringList;
 class QProgressDialog;
 namespace Kwave { class PluginWorkerThread; }
 namespace Kwave { class Plugin; }
+namespace Kwave { class PluginManager; }
 
-#define KWAVE_PLUGIN(class_name,plugin_name,author_name) \
-    extern "C" const char *name    KDE_EXPORT = plugin_name; \
-    extern "C" const char *author  KDE_EXPORT = author_name; \
-    extern "C" Kwave::Plugin *load(const PluginContext *c) KDE_EXPORT; \
-    extern "C" Kwave::Plugin *load(const PluginContext *c) { \
-	class_name *np = (c) ? new class_name(*c) : 0; \
-	return np; \
-    } \
-    extern "C" const char *version KDE_EXPORT = "2.1"
+#define KWAVE_PLUGIN(class_name,plugin_name,__version__,author_name)         \
+                                                                             \
+    extern "C" Kwave::Plugin *load(const PluginContext *c) KDE_EXPORT;       \
+    extern "C" const char    *name                         KDE_EXPORT;       \
+    extern "C" const char    *author                       KDE_EXPORT;       \
+    extern "C" const char    *version                      KDE_EXPORT;       \
+                                                                             \
+    extern "C" Kwave::Plugin *load(const PluginContext *c) {                 \
+	class_name *np = (c) ? new class_name(*c) : 0;                       \
+	return np;                                                           \
+    }                                                                        \
+                                                                             \
+    const char *name    = plugin_name;                                       \
+    const char *author  = author_name;                                       \
+    const char *version = __version__
 
 namespace Kwave {
 
@@ -176,7 +183,7 @@ namespace Kwave {
 	/**
 	* Returns a reference to the manager of this plugin.
 	*/
-	PluginManager &manager();
+	Kwave::PluginManager &manager();
 
 	/** Returns a reference to the current signal manager */
 	SignalManager &signalManager();
@@ -210,7 +217,7 @@ namespace Kwave {
 	* Returns the sample rate of the current signal. If no signal is
 	* present the return value will be zero.
 	*/
-	virtual qreal signalRate();
+	virtual double signalRate();
 
 	/**
 	* Returns an array of indices of currently selected channels.
@@ -258,7 +265,7 @@ namespace Kwave {
 	* @param percent the zoom factor to be formated, a value of "100.0"
 	*             means "100%", "0.1" means "0.1%" and so on.
 	*/
-	static QString zoom2string(qreal percent);
+	static QString zoom2string(double percent);
 
 	/**
 	* Converts a time in milliseconds into a string. Times below one
@@ -273,7 +280,7 @@ namespace Kwave {
 	*                  must be >= 3 !
 	* @return time formatted as user-readable string
 	*/
-	static QString ms2string(qreal ms, int precision = 6);
+	static QString ms2string(double ms, int precision = 6);
 
 	/**
 	* Converts the given number into a string with the current locale's
@@ -285,7 +292,7 @@ namespace Kwave {
 
     protected:
 
-	friend class ::PluginManager;
+	friend class Kwave::PluginManager;
 
 	/**
 	* Gets called to execute the plugin's run function in a separate
@@ -327,6 +334,12 @@ namespace Kwave {
 	/** can be used by plugins to execute toplevel commands */
 	void sigCommand(const QString &command);
 
+	/**
+	 * Sets the text of the progress dialog
+	 * @param text new progress bar text, already be localized
+	 */
+	void setProgressText(const QString &text);
+
     public slots:
 
 	/**
@@ -357,6 +370,14 @@ namespace Kwave {
 	/** decrements the usage counter */
 	void release();
 
+    private slots:
+
+	/** closes the progress dialog and the confirm/cancel proxy */
+	void closeProgressDialog(Kwave::Plugin *);
+
+	/** updates the progress bar, triggered by timer */
+	void updateProgressTick();
+
     protected:
 
 	friend class Kwave::PluginWorkerThread;
@@ -402,6 +423,15 @@ namespace Kwave {
 	/** Mutex for locking the usage counter */
 	QMutex m_usage_lock;
 
+	/** timer for updating the progres dialog */
+	QTimer m_progress_timer;
+
+	/** latest progress value [percent] */
+	int m_current_progress;
+
+	/** Mutex for locking the progress bar */
+	QMutex m_progress_lock;
+
     };
 
 }
