@@ -50,14 +50,14 @@
 
 //***************************************************************************
 //***************************************************************************
-PluginManager::PluginDeleter::PluginDeleter(Kwave::Plugin *plugin,
-                                            void *handle)
+Kwave::PluginManager::PluginDeleter::PluginDeleter(Kwave::Plugin *plugin,
+                                                   void *handle)
   :QObject(), m_plugin(plugin), m_handle(handle)
 {
 }
 
 //***************************************************************************
-PluginManager::PluginDeleter::~PluginDeleter()
+Kwave::PluginManager::PluginDeleter::~PluginDeleter()
 {
     // delete the plugin, this should also remove everything it has allocated
     if (m_plugin) delete m_plugin;
@@ -71,14 +71,15 @@ PluginManager::PluginDeleter::~PluginDeleter()
 
 // static initializers
 
-QMap<QString, QString> PluginManager::m_plugin_files;
+QMap<QString, QString> Kwave::PluginManager::m_plugin_files;
 
-PluginManager::PluginList PluginManager::m_unique_plugins;
+Kwave::PluginManager::PluginList Kwave::PluginManager::m_unique_plugins;
 
 static QList<PlaybackDeviceFactory *> m_playback_factories;
 
 //***************************************************************************
-PluginManager::PluginManager(QWidget *parent, SignalManager &signal_manager)
+Kwave::PluginManager::PluginManager(QWidget *parent,
+                                    SignalManager &signal_manager)
     :m_loaded_plugins(), m_running_plugins(),
      m_parent_widget(parent), m_signal_manager(signal_manager)
 {
@@ -94,13 +95,14 @@ PluginManager::PluginManager(QWidget *parent, SignalManager &signal_manager)
 	    // the plugin (when it is in use but the plugin manager
 	    // who has created it is already finished)
 	    connect(p,    SIGNAL(sigClosed(Kwave::Plugin *)),
-	            this, SLOT(pluginClosed(Kwave::Plugin *)));
+	            this, SLOT(pluginClosed(Kwave::Plugin *)),
+	            Qt::QueuedConnection);
 	}
     }
 }
 
 //***************************************************************************
-PluginManager::~PluginManager()
+Kwave::PluginManager::~PluginManager()
 {
     // inform all plugins and client windows that we close now
     emit sigClosed();
@@ -131,7 +133,7 @@ PluginManager::~PluginManager()
 }
 
 //***************************************************************************
-void PluginManager::loadAllPlugins()
+void Kwave::PluginManager::loadAllPlugins()
 {
     // Try to load all plugins. This has to be called only once per
     // instance of the main window!
@@ -159,7 +161,7 @@ void PluginManager::loadAllPlugins()
 }
 
 //***************************************************************************
-Kwave::Plugin *PluginManager::loadPlugin(const QString &name)
+Kwave::Plugin *Kwave::PluginManager::loadPlugin(const QString &name)
 {
 
     // first find out if the plugin is already loaded and persistent
@@ -289,7 +291,8 @@ Kwave::Plugin *PluginManager::loadPlugin(const QString &name)
 }
 
 //***************************************************************************
-int PluginManager::executePlugin(const QString &name, QStringList *params)
+int Kwave::PluginManager::executePlugin(const QString &name,
+                                        QStringList *params)
 {
     QString command;
     int result = 0;
@@ -307,7 +310,7 @@ int PluginManager::executePlugin(const QString &name, QStringList *params)
 	result = plugin->start(*params);
 
 	// maybe the start() function has called close() ?
-	if (!m_loaded_plugins.contains(plugin)) {
+	if (!plugin->isPersistent() && !m_loaded_plugins.contains(plugin)) {
 	    qDebug("PluginManager: plugin closed itself in start()"); // ###
 	    result = -1;
 	    plugin = 0;
@@ -360,7 +363,7 @@ int PluginManager::executePlugin(const QString &name, QStringList *params)
 }
 
 //***************************************************************************
-bool PluginManager::onePluginRunning()
+bool Kwave::PluginManager::onePluginRunning()
 {
     foreach (KwavePluginPointer plugin, m_loaded_plugins)
 	if (plugin && plugin->isRunning()) return true;
@@ -368,7 +371,7 @@ bool PluginManager::onePluginRunning()
 }
 
 //***************************************************************************
-void PluginManager::sync()
+void Kwave::PluginManager::sync()
 {
     while (onePluginRunning()) {
 	pthread_yield();
@@ -376,7 +379,7 @@ void PluginManager::sync()
 }
 
 //***************************************************************************
-int PluginManager::setupPlugin(const QString &name)
+int Kwave::PluginManager::setupPlugin(const QString &name)
 {
     // load the plugin
     Kwave::Plugin* plugin = loadPlugin(name);
@@ -402,14 +405,14 @@ int PluginManager::setupPlugin(const QString &name)
 }
 
 //***************************************************************************
-FileInfo &PluginManager::fileInfo()
+FileInfo &Kwave::PluginManager::fileInfo()
 {
     return m_signal_manager.fileInfo();
 }
 
 //***************************************************************************
-QStringList PluginManager::loadPluginDefaults(const QString &name,
-	const QString &version)
+QStringList Kwave::PluginManager::loadPluginDefaults(const QString &name,
+                                                     const QString &version)
 {
     QString def_version;
     QString section("plugin ");
@@ -439,9 +442,9 @@ QStringList PluginManager::loadPluginDefaults(const QString &name,
 }
 
 //***************************************************************************
-void PluginManager::savePluginDefaults(const QString &name,
-                                       const QString &version,
-                                       QStringList &params)
+void Kwave::PluginManager::savePluginDefaults(const QString &name,
+                                              const QString &version,
+                                              QStringList &params)
 {
     QString section("plugin ");
     section += name;
@@ -457,50 +460,51 @@ void PluginManager::savePluginDefaults(const QString &name,
 }
 
 //***************************************************************************
-unsigned int PluginManager::signalLength()
+unsigned int Kwave::PluginManager::signalLength()
 {
     return m_signal_manager.length();
 }
 
 //***************************************************************************
-double PluginManager::signalRate()
+double Kwave::PluginManager::signalRate()
 {
     return m_signal_manager.rate();
 }
 
 //***************************************************************************
-const QList<unsigned int> PluginManager::selectedTracks()
+const QList<unsigned int> Kwave::PluginManager::selectedTracks()
 {
     return m_signal_manager.selectedTracks();
 }
 
 //***************************************************************************
-unsigned int PluginManager::selectionStart()
+unsigned int Kwave::PluginManager::selectionStart()
 {
     return m_signal_manager.selection().first();
 }
 
 //***************************************************************************
-unsigned int PluginManager::selectionEnd()
+unsigned int Kwave::PluginManager::selectionEnd()
 {
     return m_signal_manager.selection().last();
 }
 
 //***************************************************************************
-void PluginManager::selectRange(unsigned int offset, unsigned int length)
+void Kwave::PluginManager::selectRange(unsigned int offset,
+                                       unsigned int length)
 {
     m_signal_manager.selectRange(offset, length);
 }
 
 //***************************************************************************
-SampleWriter *PluginManager::openSampleWriter(unsigned int track,
+SampleWriter *Kwave::PluginManager::openSampleWriter(unsigned int track,
 	InsertMode mode, unsigned int left, unsigned int right)
 {
     return m_signal_manager.openSampleWriter(track, mode, left, right, true);
 }
 
 //***************************************************************************
-Kwave::SampleSink *PluginManager::openMultiTrackPlayback(
+Kwave::SampleSink *Kwave::PluginManager::openMultiTrackPlayback(
     unsigned int tracks, const QString *name)
 {
     QString device_name;
@@ -529,25 +533,25 @@ Kwave::SampleSink *PluginManager::openMultiTrackPlayback(
 }
 
 //***************************************************************************
-PlaybackController &PluginManager::playbackController()
+PlaybackController &Kwave::PluginManager::playbackController()
 {
     return m_signal_manager.playbackController();
 }
 
 //***************************************************************************
-void PluginManager::enqueueCommand(const QString &command)
+void Kwave::PluginManager::enqueueCommand(const QString &command)
 {
     emit sigCommand(command);
 }
 
 //***************************************************************************
-void PluginManager::signalClosed()
+void Kwave::PluginManager::signalClosed()
 {
     emit sigClosed();
 }
 
 //***************************************************************************
-void PluginManager::pluginClosed(Kwave::Plugin *p)
+void Kwave::PluginManager::pluginClosed(Kwave::Plugin *p)
 {
     Q_ASSERT(p);
     Q_ASSERT(!m_loaded_plugins.isEmpty() || !m_unique_plugins.isEmpty());
@@ -570,7 +574,7 @@ void PluginManager::pluginClosed(Kwave::Plugin *p)
 }
 
 //***************************************************************************
-void PluginManager::pluginStarted(Kwave::Plugin *p)
+void Kwave::PluginManager::pluginStarted(Kwave::Plugin *p)
 {
     Q_ASSERT(p);
     if (!p) return;
@@ -584,7 +588,7 @@ void PluginManager::pluginStarted(Kwave::Plugin *p)
 }
 
 //***************************************************************************
-void PluginManager::pluginDone(Kwave::Plugin *p)
+void Kwave::PluginManager::pluginDone(Kwave::Plugin *p)
 {
     Q_ASSERT(p);
     if (!p) return;
@@ -597,7 +601,7 @@ void PluginManager::pluginDone(Kwave::Plugin *p)
 }
 
 //***************************************************************************
-void PluginManager::connectPlugin(Kwave::Plugin *plugin)
+void Kwave::PluginManager::connectPlugin(Kwave::Plugin *plugin)
 {
     Q_ASSERT(plugin);
     if (!plugin) return;
@@ -608,17 +612,19 @@ void PluginManager::connectPlugin(Kwave::Plugin *plugin)
     }
 
     connect(plugin, SIGNAL(sigClosed(Kwave::Plugin *)),
-	    this, SLOT(pluginClosed(Kwave::Plugin *)));
+	    this, SLOT(pluginClosed(Kwave::Plugin *)),
+	     Qt::QueuedConnection);
 
     connect(plugin, SIGNAL(sigRunning(Kwave::Plugin *)),
 	    this, SLOT(pluginStarted(Kwave::Plugin *)));
 
     connect(plugin, SIGNAL(sigDone(Kwave::Plugin *)),
-	    this, SLOT(pluginDone(Kwave::Plugin *)));
+	    this, SLOT(pluginDone(Kwave::Plugin *)),
+	    Qt::QueuedConnection);
 }
 
 //***************************************************************************
-void PluginManager::disconnectPlugin(Kwave::Plugin *plugin)
+void Kwave::PluginManager::disconnectPlugin(Kwave::Plugin *plugin)
 {
     Q_ASSERT(plugin);
     if (!plugin) return;
@@ -639,13 +645,13 @@ void PluginManager::disconnectPlugin(Kwave::Plugin *plugin)
 }
 
 //***************************************************************************
-void PluginManager::setSignalName(const QString &name)
+void Kwave::PluginManager::setSignalName(const QString &name)
 {
     emit sigSignalNameChanged(name);
 }
 
 //***************************************************************************
-void PluginManager::findPlugins()
+void Kwave::PluginManager::findPlugins()
 {
     if (!m_plugin_files.isEmpty()) {
 	// this is not the first call -> bail out
@@ -692,14 +698,14 @@ void PluginManager::findPlugins()
 }
 
 //***************************************************************************
-void PluginManager::registerPlaybackDeviceFactory(
+void Kwave::PluginManager::registerPlaybackDeviceFactory(
     PlaybackDeviceFactory *factory)
 {
     m_playback_factories.append(factory);
 }
 
 //***************************************************************************
-void PluginManager::unregisterPlaybackDeviceFactory(
+void Kwave::PluginManager::unregisterPlaybackDeviceFactory(
     PlaybackDeviceFactory *factory)
 {
     m_playback_factories.removeAll(factory);
