@@ -52,6 +52,14 @@ static inline float sample2float(const sample_t s) {
 	static_cast<float>(s) / static_cast<float>(1 << (SAMPLE_BITS-1)));
 }
 
+/**
+ * Simple conversion from sample_t to double
+ */
+static inline double sample2double(const sample_t s) {
+    return static_cast<double>(
+	static_cast<double>(s) / static_cast<double>(1 << (SAMPLE_BITS-1)));
+}
+
 /**
  * Simple conversion from double to sample_t
  */
