@@ -21,19 +21,19 @@
 
 #include "config.h"
 
+#include <QList>
 #include <QObject>
 #include <QTime>
 
 #include <kdemacros.h>
 
 #include "libkwave/InsertMode.h"
+#include "libkwave/ReaderMode.h"
 #include "libkwave/Sample.h"
+#include "libkwave/Stripe.h"
 #include "libkwave/KwaveSampleArray.h"
 #include "libkwave/KwaveSampleSource.h"
 
-class Stripe;
-class Track;
-
 class KDE_EXPORT SampleReader: public Kwave::SampleSource
 {
     Q_OBJECT
@@ -41,13 +41,15 @@ public:
 
     /**
      * Constructor. Creates a stream for reading samples from a track.
-     * @param track reference to the track from which we want to read
+     * @param mode the reader mode, see Kwave::ReaderMode
+     * @param stripes list of stripes which contain data in the desired range
      * @param left start of the input (only useful in insert and
      *             overwrite mode)
      * @param right end of the input (only useful with overwrite mode)
      * @see InsertMode
      */
-    SampleReader(Track &track, unsigned int left, unsigned int right);
+    SampleReader(Kwave::ReaderMode mode, QList<Stripe> stripes,
+                 unsigned int left, unsigned int right);
 
     /** Destructor */
     virtual ~SampleReader();
@@ -148,10 +150,27 @@ protected:
     /** Fills the sample buffer */
     void fillBuffer();
 
+    /**
+     * Read a block of samples, with padding if necessary.
+     *
+     * @param offset position where to start the read operation
+     * @param buffer receives the samples
+     * @param buf_offset offset within the buffer
+     * @param length number of samples to read
+     * @return number of read samples
+     */
+    unsigned int readSamples(unsigned int offset,
+                             Kwave::SampleArray &buffer,
+                             unsigned int buf_offset,
+                             unsigned int length);
+
 private:
 
-    /** the track to which we belong */
-    Track &m_track;
+    /** operation mode of the reader, see Kwave::ReaderMode */
+    Kwave::ReaderMode m_mode;
+
+    /** list of stipes in range */
+    QList<Stripe> m_stripes;
 
     /**
      * Current sample position, related to the source of the samples. Does
@@ -179,6 +198,9 @@ private:
     /** timer for limiting the number of progress signals per second */
     QTime m_progress_time;
 
+    /** last seek position, needed in SinglePassReverse mode */
+    unsigned int m_last_seek_pos;
+
 };
 
 #endif /* _SAMPLE_READER_H_ */
