@@ -21,6 +21,8 @@
 #include <limits.h>
 #include <math.h>
 
+#include <QApplication>
+#include <QCursor>
 #include <QFile>
 #include <QFileInfo>
 #include <QMutexLocker>
@@ -46,7 +48,6 @@
 #include "libkwave/MultiTrackWriter.h"
 #include "libkwave/Parser.h"
 #include "libkwave/Sample.h"
-#include "libkwave/SampleFormat.h"
 #include "libkwave/SampleReader.h"
 #include "libkwave/SampleWriter.h"
 #include "libkwave/Signal.h"
@@ -340,7 +341,7 @@ int SignalManager::save(const KUrl &url, bool selection)
 	QString filename = url.path();
 	QFile dst(filename);
 
-	MultiTrackReader src(*this,
+	MultiTrackReader src(Kwave::SinglePassForward, *this,
 	    (selection) ? selectedTracks() : allTracks(),
 	    ofs, ofs+len-1);
 
@@ -379,7 +380,7 @@ int SignalManager::save(const KUrl &url, bool selection)
 	    m_file_info.set(INF_CREATION_DATE, value);
 	}
 
-	//prepare and show the progress dialog
+	// prepare and show the progress dialog
 	FileProgress *dialog = new FileProgress(m_parent_widget,
 	    filename, m_file_info.tracks()*m_file_info.length()*
 	    (m_file_info.bits() >> 3),
@@ -429,6 +430,7 @@ int SignalManager::save(const KUrl &url, bool selection)
 	    res = -1;
 	}
 
+	qApp->flush();
 	delete encoder;
 	if (dialog) {
 	    if (dialog->isCanceled()) {
@@ -437,7 +439,8 @@ int SignalManager::save(const KUrl &url, bool selection)
 		    i18n("The file has been truncated and "\
 		         "might be corrupted!"));
 	    }
-	    delete dialog;
+	    dialog->deleteLater();
+	    qApp->flush();
 	}
     } else {
 	Kwave::MessageBox::error(m_parent_widget,
@@ -470,12 +473,6 @@ void SignalManager::newSignal(unsigned int samples, double rate,
     m_file_info.clear();
     m_file_info.setRate(rate);
     m_file_info.setBits(bits);
-
-    // use "unsigned" sample formats per default for <= 8 bits/sample formats
-    if (bits <= 8) {
-	const SampleFormat format(SampleFormat::Unsigned);
-	m_file_info.set(INF_SAMPLE_FORMAT, QVariant(format.toInt()));
-    }
     emit labelsChanged(labels());
 
     // now the signal is considered not to be empty
@@ -582,7 +579,8 @@ SampleWriter *SignalManager::openSampleWriter(unsigned int track,
     UndoTransactionGuard guard(*this, 0);
     startUndoTransaction();
     QObject::connect(writer, SIGNAL(destroyed()),
-                     this,   SLOT(closeUndoTransaction()));
+                     this,   SLOT(closeUndoTransaction()),
+                     Qt::QueuedConnection);
 
     // create an undo action for the modification of the samples
     UndoAction *undo = 0;
@@ -744,6 +742,7 @@ void SignalManager::slotTrackInserted(unsigned int index,
 	Track *track)
 {
     setModified(true);
+    m_file_info.setTracks(tracks());
     emit sigTrackInserted(index, track);
     emitStatusInfo();
 }
@@ -752,6 +751,7 @@ void SignalManager::slotTrackInserted(unsigned int index,
 void SignalManager::slotTrackDeleted(unsigned int index)
 {
     setModified(true);
+    m_file_info.setTracks(tracks());
     emit sigTrackDeleted(index);
     emitStatusInfo();
 }
@@ -1127,7 +1127,7 @@ bool SignalManager::registerUndoAction(UndoAction *action)
     }
 
     // check if the undo action is too large
-    unsigned int limit_mb     = MemoryManager::instance().undoLimit();
+    unsigned int limit_mb     = Kwave::MemoryManager::instance().undoLimit();
     unsigned int needed_size  = action->undoSize();
     unsigned int needed_mb = needed_size  >> 20;
     if (needed_mb > limit_mb) {
@@ -1193,7 +1193,8 @@ unsigned int SignalManager::usedUndoRedoMemory()
 void SignalManager::freeUndoMemory(unsigned int needed)
 {
     unsigned int size = usedUndoRedoMemory() + needed;
-    unsigned int undo_limit = MemoryManager::instance().undoLimit() << 20;
+    unsigned int undo_limit =
+	Kwave::MemoryManager::instance().undoLimit() << 20;
 
     // remove old undo actions if not enough free memory
     while (!m_undo_buffer.isEmpty() && (size > undo_limit)) {
@@ -1261,7 +1262,8 @@ void SignalManager::undo()
     m_undo_enabled = false;
 
     // get free memory for redo
-    unsigned int undo_limit = MemoryManager::instance().undoLimit() << 20;
+    unsigned int undo_limit =
+	Kwave::MemoryManager::instance().undoLimit() << 20;
     unsigned int redo_size = undo_transaction->redoSize();
     unsigned int undo_size = undo_transaction->undoSize();
     UndoTransaction *redo_transaction = 0;
@@ -1285,6 +1287,9 @@ void SignalManager::undo()
 	qDebug("SignalManager::undo(): redo buffer flushed!");
     }
 
+    // set hourglass cursor
+    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+
     // execute all undo actions and store the resulting redo
     // actions into the redo transaction
     while (!undo_transaction->isEmpty()) {
@@ -1357,6 +1362,9 @@ void SignalManager::undo()
 
     // finished / buffers have changed, emit new undo/redo info
     emitUndoRedoInfo();
+
+    // remove hourglass
+    QApplication::restoreOverrideCursor();
 }
 
 //***************************************************************************
@@ -1380,7 +1388,8 @@ void SignalManager::redo()
     m_undo_enabled = false;
 
     // get free memory for undo
-    unsigned int undo_limit = MemoryManager::instance().undoLimit() << 20;
+    unsigned int undo_limit =
+	Kwave::MemoryManager::instance().undoLimit() << 20;
     unsigned int undo_size = redo_transaction->undoSize();
     unsigned int redo_size = redo_transaction->redoSize();
     UndoTransaction *undo_transaction = 0;
@@ -1406,6 +1415,9 @@ void SignalManager::redo()
 	m_undo_buffer.append(undo_transaction);
     }
 
+    // set hourglass cursor
+    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+
     // execute all redo actions and store the resulting undo
     // actions into the undo transaction
     while (!redo_transaction->isEmpty()) {
@@ -1456,6 +1468,9 @@ void SignalManager::redo()
 
     // finished / buffers have changed, emit new undo/redo info
     emitUndoRedoInfo();
+
+    // remove hourglass
+    QApplication::restoreOverrideCursor();
 }
 
 //***************************************************************************
