@@ -22,15 +22,19 @@
 #include <sys/types.h>  // for size_t
 
 #include <QFile>
+#include <QTemporaryFile>
 
-class MemoryManager;
 class QString;
 
 class SwapFile
 {
 public:
-    /** Constructor */
-    SwapFile();
+    /**
+     * Constructor
+     * @param name full path to the swap file, actually a template
+     *             that <b>must</b> contain 6 "X" characters at the end!
+     */
+    SwapFile(const QString &name);
 
     /** Destructor */
     virtual ~SwapFile();
@@ -44,35 +48,34 @@ public:
      *                 that <b>must</b> contain 6 "X" characters at the end!
      * @return true if succeeded, false if failed
      */
-    bool allocate(size_t size, const QString &filename);
+    bool allocate(size_t size);
 
     /**
      * Returns the address of the allocated memory or 0 if
      * nothing has been allocated.
      */
-    inline void *address()   { return m_address; }
+    inline void *address() const   { return m_address; }
 
     /**
      * Returns the size of the allocated memory or 0 if
      * nothing has been allocated.
      */
-    inline size_t size()     { return m_size; }
+    inline size_t size() const     { return m_size; }
+
+    /** returns the map count */
+    inline int mapCount() const    { return m_map_count; }
 
     /**
      * Returns the size of one storage unit in bytes
      */
-    inline size_t pagesize() { return m_pagesize; }
+    inline size_t pagesize() const { return m_pagesize; }
 
     /**
      * Resizes the allocated swap file.
      * @param size the new size
-     * @return pointer to this object or 0 if failed
+     * @return true if successful or false if failed
      */
-    SwapFile *resize(size_t size);
-
-protected:
-
-    friend class MemoryManager;
+    bool resize(size_t size);
 
     /**
      * Map the memory and return the physical address.
@@ -84,10 +87,9 @@ protected:
     /**
      * Unmap a memory area, previously mapped with map()
      *
-     * @param block pointer to the previously mapped block
-     * @param pointer to this
+     * @return current reference count
      */
-    void *unmap();
+    int unmap();
 
     /**
      * Read bytes into a buffer
@@ -119,18 +121,21 @@ private:
 
 private:
 
-    /** File used for swapping */
-    QFile m_file;
+    /** file used for swapping */
+    QTemporaryFile m_file;
 
-    /** Address of the allocated virtual memory or 0 */
+    /** address of the allocated virtual memory or 0 */
     void *m_address;
 
-    /** Number of allocated bytes or 0 */
+    /** number of allocated bytes or 0 */
     size_t m_size;
 
     /** size of one storage unit */
     size_t m_pagesize;
 
+    /** reference count for mmap [0...N] */
+    int m_map_count;
+
 };
 
 #endif /* _SWAP_FILE_H_ */
