@@ -68,8 +68,8 @@ int UndoDeleteAction::redoSize()
 //***************************************************************************
 bool UndoDeleteAction::store(SignalManager &manager)
 {
-    MultiTrackReader reader(manager, m_track_list,
-	m_offset, m_offset + m_length - 1);
+    MultiTrackReader reader(Kwave::SinglePassForward, manager,
+	m_track_list, m_offset, m_offset + m_length - 1);
 
     // encode the data that will be deleted into a Kwave::MimeData container
     if (!m_mime_data.encode(m_parent_widget, reader, manager.fileInfo())) {
