@@ -33,7 +33,7 @@
 #include "AmplifyFreePlugin.h"
 #include "AmplifyFreeDialog.h"
 
-KWAVE_PLUGIN(AmplifyFreePlugin,"amplifyfree","Thomas Eschenbacher");
+KWAVE_PLUGIN(AmplifyFreePlugin,"amplifyfree","2.1","Thomas Eschenbacher");
 
 //***************************************************************************
 AmplifyFreePlugin::AmplifyFreePlugin(const PluginContext &context)
@@ -141,7 +141,8 @@ void AmplifyFreePlugin::run(QStringList params)
     unsigned int tracks = selectedTracks().count();
 
     // create all objects
-    MultiTrackReader source(signalManager(), selectedTracks(), first, last);
+    MultiTrackReader source(Kwave::SinglePassForward,
+	signalManager(), selectedTracks(), first, last);
     Kwave::CurveStreamAdapter curve(m_curve, input_length);
     MultiTrackWriter sink(signalManager(), selectedTracks(), Overwrite,
 	first, last);
@@ -169,7 +170,7 @@ void AmplifyFreePlugin::run(QStringList params)
     // connect the progress dialog
     connect(&sink, SIGNAL(progress(unsigned int)),
 	    this,  SLOT(updateProgress(unsigned int)),
-	    Qt::QueuedConnection);
+	     Qt::BlockingQueuedConnection);
 
     // transport the samples
     qDebug("AmplifyFreePlugin: filter started...");
