@@ -32,6 +32,7 @@
 #include "libkwave/MessageBox.h"
 #include "libkwave/MultiTrackReader.h"
 #include "libkwave/Sample.h"
+#include "libkwave/SampleFormat.h"
 #include "libkwave/SampleReader.h"
 #include "libkwave/VirtualAudioFile.h"
 
@@ -68,7 +69,7 @@ Encoder *WavEncoder::instance()
 /***************************************************************************/
 QList<FileProperty> WavEncoder::supportedProperties()
 {
-    return m_property_map.values();
+    return m_property_map.properties();
 }
 
 /***************************************************************************/
@@ -81,7 +82,7 @@ void WavEncoder::fixAudiofileBrokenHeaderBug(QIODevice &dst, FileInfo &info,
                       info.get(INF_COMPRESSION).toInt() :
 	    AF_COMPRESSION_NONE;
     if (compression != AF_COMPRESSION_NONE) {
-	qWarning("WARNING: libaudiofile might have produces a broken header!");
+	qWarning("WARNING: libaudiofile might have produced a broken header!");
 	return;
     }
 
@@ -353,6 +354,23 @@ bool WavEncoder::encode(QWidget *widget, MultiTrackReader &src,
 	}
     }
 
+    // check for unsupported bits/sample format combinations
+    if (compression == AF_COMPRESSION_NONE) {
+	if ((bits <= 8) && (sample_format != AF_SAMPFMT_UNSIGNED)) {
+	    const SampleFormat format(SampleFormat::Unsigned);
+	    info.set(INF_SAMPLE_FORMAT, QVariant(format.toInt()));
+
+	    sample_format = AF_SAMPFMT_UNSIGNED;
+	    qDebug("auto-switching to unsigned format");
+	} else if ((bits > 8) && (sample_format != AF_SAMPFMT_TWOSCOMP)) {
+	    const SampleFormat format(SampleFormat::Signed);
+	    info.set(INF_SAMPLE_FORMAT, QVariant(format.toInt()));
+
+	    sample_format = AF_SAMPFMT_TWOSCOMP;
+	    qDebug("auto-switching to signed format");
+	}
+    }
+
     // open the output device
     if (!dst.open(QIODevice::ReadWrite | QIODevice::Truncate)) {
 	Kwave::MessageBox::error(widget,
@@ -419,8 +437,6 @@ bool WavEncoder::encode(QWidget *widget, MultiTrackReader &src,
 	AF_SAMPFMT_TWOSCOMP, SAMPLE_STORAGE_BITS);
 
     // allocate a buffer for input data
-    const unsigned int frame_size = static_cast<const unsigned int>(
-	    afGetFrameSize(fh, AF_DEFAULT_TRACK, 1));
     const unsigned int virtual_frame_size = static_cast<const unsigned int>(
 	    afGetVirtualFrameSize(fh, AF_DEFAULT_TRACK, 1));
     const unsigned int buffer_frames = (8*1024);
@@ -478,7 +494,7 @@ bool WavEncoder::encode(QWidget *widget, MultiTrackReader &src,
 
     // due to a buggy implementation of libaudiofile
     // we have to fix up the length of the "data" and the "RIFF" chunk
-    fixAudiofileBrokenHeaderBug(dst, info, frame_size);
+    fixAudiofileBrokenHeaderBug(dst, info, (bits * tracks) >> 3);
 
     // put the properties into the INFO chunk
     writeInfoChunk(dst, info);
