@@ -56,11 +56,12 @@
 
 #include "PlayBack-OSS.h"
 #include "PlayBack-ALSA.h"
+#include "PlayBack-Phonon.h"
 
 #include "PlayBackDialog.h"
 #include "PlayBackPlugin.h"
 
-KWAVE_PLUGIN(PlayBackPlugin,"playback","Thomas Eschenbacher");
+KWAVE_PLUGIN(PlayBackPlugin,"playback","2.2","Thomas Eschenbacher");
 
 /** Sets the number of screen refreshes per second when in playback mode */
 #define SCREEN_REFRESHES_PER_SECOND 16
@@ -83,6 +84,9 @@ PlayBackPlugin::PlayBackPlugin(const PluginContext &context)
             &m_playback_controller, SLOT(playbackDone()));
     connect(this, SIGNAL(sigPlaybackPos(unsigned int)),
             &m_playback_controller, SLOT(updatePlaybackPos(unsigned int)));
+    connect(this, SIGNAL(sigPlaybackDone()),
+            this, SLOT(closeDevice()),
+            Qt::QueuedConnection);
 
     // register as a factory for playback devices
     manager().registerPlaybackDeviceFactory(this);
@@ -259,6 +263,11 @@ PlayBackDevice *PlayBackPlugin::createDevice(playback_method_t &method)
 		return new PlayBackALSA();
 #endif /* HAVE_ALSA_SUPPORT */
 
+#ifdef HAVE_PHONON_SUPPORT
+	    case PLAYBACK_PHONON:
+		return new PlayBackPhonon();
+#endif /* HAVE_PHONON_SUPPORT */
+
 	    default:
 		qDebug("unsupported playback method (%d)",
 		    static_cast<int>(method));
@@ -574,18 +583,20 @@ void PlayBackPlugin::run(QStringList)
     }
 
     // set up a set of sample reader (streams)
-    MultiTrackReader input(signalManager(), audible_tracks, first, last);
+    MultiTrackReader input(
+	Kwave::FullSnapshot,
+	signalManager(), audible_tracks, first, last);
 
     // create a translation matrix for mixing up/down to the desired
     // number of output channels
     Matrix<double> matrix(audible_count, out_channels);
     unsigned int x, y;
-    for (y=0; y < out_channels; y++) {
+    for (y = 0; y < out_channels; y++) {
 	unsigned int m1, m2;
 	m1 = y * audible_count;
-	m2 = (y+1) * audible_count;
+	m2 = (y + 1) * audible_count;
 
-	for (x=0; x < audible_count; x++) {
+	for (x = 0; x < audible_count; x++) {
 	    unsigned int n1, n2;
 	    n1 = x * out_channels;
 	    n2 = n1 + out_channels;
@@ -669,11 +680,6 @@ void PlayBackPlugin::run(QStringList)
 
     } while (m_playback_controller.loop() && !shouldStop());
 
-    // we are done, so close the output device
-    m_device->close();
-    delete m_device;
-    m_device = 0;
-
     // playback is done
     emit sigPlaybackDone();
 //    qDebug("PlayBackPlugin::run() done.");
@@ -711,6 +717,7 @@ void PlayBackPlugin::testPlayBack()
     // create the multi track playback sink
     Kwave::SampleSink *sink = manager().openMultiTrackPlayback(tracks);
     if (!sink) return;
+    sink->setInteractive(true);
 
     float t_period = t_sweep * tracks;
     unsigned int curve_length = static_cast<unsigned int>(t_period * rate);
@@ -778,12 +785,16 @@ void PlayBackPlugin::testPlayBack()
 	    progress->setMinimumDuration(0);
 	    progress->setMaximum(100);
 	    progress->setAutoClose(true);
-	    progress->setValue(0);
+	    progress->setValue(1);
 	    progress->setLabelText(
-		i18n("you should now hear a %1Hz test tone...\n\n"\
-		     "(if you hear clicks or dropouts, "\
-		     "please increase the buffer size "\
-		     "and try again)", static_cast<int>(freq)));
+		"<html><p><br>" +
+		i18n("you should now hear a %1Hz test tone...<br><br>"\
+		     "(if you hear clicks or dropouts, please increase<br>"\
+		     "the buffer size and try again)", static_cast<int>(freq)) +
+		"</p></html>"
+	    );
+	    progress->show();
+	    QApplication::processEvents();
 	}
     }
 
@@ -800,12 +811,14 @@ void PlayBackPlugin::testPlayBack()
 
 	if (progress) {
 	    progress->setValue((100 * time.elapsed()) / t_max);
+	    QApplication::processEvents();
 	    if (progress->wasCanceled()) break;
 	}
     }
+    sink->setInteractive(false);
 
     if (progress) delete progress;
-    if (sink)     delete sink;
+    delete sink;
 }
 
 //***************************************************************************
