@@ -236,8 +236,7 @@ void RecordALSA::detectSupportedFormats()
 	const snd_pcm_format_t *fmt = &(_known_formats[i]);
 
 	// eliminate duplicate alsa sample formats (e.g. BE/LE)
-	foreach (int it, m_supported_formats)
-	{
+	foreach (int it, m_supported_formats) {
 	    const snd_pcm_format_t *f = &_known_formats[it];
 	    if (*f == *fmt) {
 		fmt = 0;
@@ -1070,7 +1069,7 @@ void RecordALSA::scanDevices()
 		QString name = QString(
 		    i18n("card %1: ", card_name) + "|sound_card||" +
 		    i18n("device %1: ", device_name) + "|sound_subdevice"
-		).arg(card).arg(dev);
+		)/*.arg(card).arg(dev)*/;
 // 		qDebug("# '%s' -> '%s'", hw_device.data(), name.data());
 		m_device_list.insert(name, hw_device);
 	    }
