@@ -53,7 +53,7 @@
 #include "Record-ALSA.h"
 #include "Record-OSS.h"
 
-KWAVE_PLUGIN(RecordPlugin,"record","Thomas Eschenbacher");
+KWAVE_PLUGIN(RecordPlugin,"record","2.2","Thomas Eschenbacher");
 
 //***************************************************************************
 RecordPlugin::RecordPlugin(const PluginContext &context)
@@ -985,7 +985,11 @@ void RecordPlugin::stateChanged(RecordState state)
 	case REC_PAUSED:
 	case REC_DONE:
 	    // reset buffer status
-	    if (m_writers) m_writers->flush();
+	    if (m_writers) {
+		m_writers->flush();
+		delete m_writers;
+		m_writers = 0;
+	    }
 	    m_buffers_recorded = 0;
 	    m_dialog->updateBufferState(0,0);
 	    break;
