@@ -30,7 +30,7 @@
 #include "SelectRangePlugin.h"
 #include "SelectRangeDialog.h"
 
-KWAVE_PLUGIN(SelectRangePlugin,"selectrange","Thomas Eschenbacher");
+KWAVE_PLUGIN(SelectRangePlugin,"selectrange","2.1","Thomas Eschenbacher");
 
 //***************************************************************************
 SelectRangePlugin::SelectRangePlugin(const PluginContext &c)
@@ -52,7 +52,7 @@ QStringList *SelectRangePlugin::setup(QStringList &previous_params)
     interpreteParameters(previous_params);
 
     // create the setup dialog
-    qreal rate = signalRate();
+    double rate = signalRate();
     unsigned int offset = 0;
     selection(&offset, 0, false);
     unsigned int length = signalLength();
